/****************************************************************************
    PWM.c
    -----
    bungsprojekt zum Ausgeben einer analogen Spannung ueber
    Pulsweitenmodulation.
    Ueber den UART werden Zeichen empfangen.Handelt es sich um eine
    hexadezimale Ziffer ('0' bis 'A'), dann wird der entsprechende
    Zahlenwert multipliziert und als Vergleichswert in den PWM geschrieben.

    Autor:    Christian Schifferle
    Datum:    November 2002
    Software: AVR-GCC
    needed
****************************************************************************/

#include <io.h>	// Wir brauchen Zugriff auf die I/O's des Controllers

#define F_CPU            8000000      /* 8Mhz */
#define UART_BAUD_RATE      9600      /* 9600 baud */

// Berechnung des Inhalts des UBBR-Registers
#define UART_BAUD_SELECT (F_CPU/(UART_BAUD_RATE*16l)-1)

// Multiplikator fr Vergleichswert
#define MUPL	16

// Datentypdefinitionen
typedef unsigned char	BYTE;
typedef unsigned short	WORD;



int main (void)
/****************************************************************************
    Hier ist der Programmeinstiegspunkt.
*/
{
    register BYTE c;    // Empfangenes Zeichen
    register WORD x;	// Auszugebender Wert

    /* UART Control Register laden. Wir verwenden (immer noch)
       keine Interrupts. Hier wollen wir nur empfangen, also wird
       nur das Receiver Enable Flag gesetzt. */
    outp ((1 << RXEN), UCR);

    /* Baudrate einstellen */
    outp ((BYTE) UART_BAUD_SELECT, UBRR);          

    // Bits 3 und 4 von Port B als Ausgang konfigurieren
    outp ((1<<PINB3)|(1<<PINB4), DDRB);

    // Timer/Counter1 initialisieren
    outp ((1<<PWM10)|(1<<COM1A1), TCCR1A);
    outp ((1<<CS10), TCCR1B); // Vorzaehler = 0

    // Vergleichswert irgendwo in die Mitte setzen, damit
    // man von Beginn an gleich etwas sieht
    __outw (MUPL * 8, OCR1L);

    for (;;) {
	// Pin 4 toggeln um mit dem Oszi pruefen zu koennen
        if (bit_is_set (PORTB, PINB4)) cbi (PORTB, PINB4);
        else sbi (PORTB, PINB4);

        // Prfen, ob ein Zeichen empfangen wurde.

        if (inp (USR) & (1 << RXC)) {     // Bit RXC im USR gesetzt ?
            // Vergleichswert initialisieren auf -1, weil WORD mit 0xffff
            x = 0xffff;

            // Zeichen einlesen
            c = inp (UDR);

            // und pruefen, ob Hexziffer
            if (c >= '0' && c <= '9') {
                x = c - '0';
            }
            else if (c >= 'A' && c <= 'F') {
                x = c - 'A' + 10;
            }
            else if (c >= 'a' && c <= 'f') {
                x = c - 'a' + 10;
            }

            // Wenn eine gueltige Hexziffer empfangen wurde
            if (x != 0xffff) {
               // Wert ausgeben
               __outw (MUPL * x, OCR1L);
            }
        }
    }

} /* end of main */
